/*                      Watcher V.1
 *                     Matthew Paley
 *                     mtp@uk.ac.ukc
 *                      August 1991
 *
 * This program checks for suspicious login activity - either consecutive
 * failures at a terminal or by on same account.
 * It must be run with auth authorization - usually by root.
*/

#include "bigb.h"
#include <fcntl.h>
#include <time.h>
#include <string.h>
#include <signal.h>

/* Just to remove some warnings on compile */
void         exit(int);
PRIVATE void daemonise(void);

main(argc,argv)
int     argc;
char    *argv[];
{
    /* Initialise the protected password entry routines */
    set_auth_parameters(argc,argv);

    /* These will be set in read_config() */
    error_display_chan  = FNULL;    /* Send <BLEEP> WARNING: messages here */
    error_file_chan     = FNULL;    /* Send plain messages here */

    /* Read the various options from the config file */
    read_config();

    /* Read the parameter files (users to check/not check ... ) */
    get_params();

    /* Go into the background */
    daemonise();

    /* Update the options on SIGHUP's */
    signal(SIGHUP,update_data);

    /* Stop ancient events being noticed the first time through */
    last_checked=time(&last_checked);

    /* The main loop */
    do {
        if (max_term_failures >= 0)
            check_terminals();
        if (max_login_failures >= 0)
            check_users();
        if (check_root_selected == TRUE)
            check_root();

        /* Ensure the outputs are kept up to date */
        fflush(error_display_chan);
        fflush(error_file_chan);
    
        /* Note the time so old events can be ignored */
        last_checked=time(&last_checked);

        /* Sleep until the next check is due */
        sleep(sleeptime);
    } while (TRUE);
}

/* Put this process in the background */
void daemonise(void)
{
    int     pid;

    if ((pid = fork()) < 0) {
        perror("Cannot fork");
        exit(errno);
    }

    if (pid != 0) {
        /* This is the parent */
        puts("bigb ( is watching you ) : the breakin alerter started");
        exit(0);    /* Suicide */
    }

    /* This is now the child */

    /* Start a new process group with this as the master */
    setpgrp();
}
