/* Check for repeated failures at a terminal */

/* This may look unnecessarily complicated but I am trying to avoid using
 * getpwtcent() and the linked list searcher more than is needed.
 */
 
#include "bigb.h"
#include <sys/stat.h>
#include <time.h>

PRIVATE void check_all_t(void);
PRIVATE void check_specific_t(void);
PRIVATE void check_all_except_t(void);
PRIVATE void check_some_except_t(void);
PRIVATE void check_this_term(struct pr_term *);

#ifdef QUICK
PRIVATE int check_terminal_changes(void);
#endif

PUBLIC void check_terminals(void)
{
#ifdef QUICK
    if (check_terminal_changes() == FALSE)
        return;     /* Nothing has happened since last check */
#endif

    switch(terminal_check) {
        case CHECK_ALL      : check_all_t();
                            break;
        case CHECK_SPECIFIC : check_specific_t();
                            break;
        case CHECK_EXCLUDE  : check_all_except_t();
                            break;
        case CHECK_NONE     : /* Nothing required */
                            break;
        default             : check_some_except_t();
                            break;
    }
}

/* Check every terminal the system knows of */
PRIVATE void check_all_t(void)
{
    struct pr_term  *term;

    while ((term=getprtcent()) != (struct pr_term *) NULL)
        check_this_term(term);
    endprtcent();
}

/* Check a specific list of terminals */
PRIVATE void check_specific_t(void)
{
    struct member   *ptr1;
    struct pr_term  *term;

    ptr1 = terminals_to_check;
    if (ptr1 == MNULL)
        return; /* No terminals need to be checked */

    do {
        if ((term=getprtcnam(ptr1->data)) != (struct pr_term *) NULL)
            check_this_term(term);
    } while ((ptr1=ptr1->next) != MNULL);
    endprtcent();
}

/* Check all the terminals except those specifically excluded */
PRIVATE void check_all_except_t(void)
{
    struct pr_term  *term;

    while ((term=getprtcent()) != (struct pr_term *) NULL) {
        if (!find_in_list(terminals_to_exclude,term->ufld.fd_devname))
            check_this_term(term);
    };
    endprtcent();
}

/* The most general case: check all specified terminals except those
 * in the exclusion file.
 */
PRIVATE void check_some_except_t(void)
{
    struct member   *ptr1;
    struct pr_term  *term;

    ptr1 = terminals_to_check;
    if (ptr1 == MNULL)
        return; /* No terminals need to be checked */

    do {
        if ((term=getprtcnam(ptr1->data)) != (struct pr_term *) NULL)
            if (!find_in_list(terminals_to_exclude,term->ufld.fd_devname))
                check_this_term(term);
    } while ((ptr1=ptr1->next) != MNULL);
    endprtcent();
}

/* Check the terminal data given */
PRIVATE void check_this_term(struct pr_term *term)
{
    struct t_field  *data;
    struct t_flag   *flags;
    time_t          last_used;
    
    data  = &(term->ufld);
    flags = &(term->uflg);

    if (!flags->fg_slogin && !flags->fg_ulogin)
        return;     /* Never been used */

    last_used = (data->fd_slogin > data->fd_ulogin)?
        data->fd_slogin : data->fd_ulogin;
        
    if (last_checked > last_used)
        return;     /* Not used since last check */
    
    if (data->fd_nlogins >= max_term_failures) {
        sprintf(temp, "%s: %d consecutive failures on %s %s",
            date(last_used),
            data->fd_nlogins,
            data->fd_devname,
            (flags->fg_label)? data->fd_label:"");
        print_warning(temp);
    }
}

#ifdef QUICK

/* Check to see if any /entry in the terminal database has been changed since
 * they were last checked by this program.  This is done by looking at the
 * modification time of the file.
*/
PRIVATE int check_terminal_changes(void)
{
    struct stat buf;

    if (stat(TERMINALFILE,&buf) == 0) {
        if (last_checked > buf.st_mtime)
            return(FALSE);      /* No change */
        else
            return(TRUE);       /* Something has changed */
    }
    return(TRUE);   /* Something strange is going on */
}

#endif
