/* Data is stored in linked lists - this file contains the list processing
 * routines and also the routine that reads data from a file and puts it
 * in a list.
*/

#include "bigb.h"
#include <memory.h>
#include <string.h>

/* Read in a list of strings into a linked list pointer to by start */
PUBLIC int read_list(struct member **start, FILE *fp)
{
    char            buffer[MAX_LEN+1];
    struct member   *ptr1,*ptr2;
    int             num = 0;
    
    *start = (struct member *) malloc(sizeof(struct member));
    ptr1 = *start;
    ptr1->previous = MNULL;
    
    while (fgets(buffer, MAX_LEN, fp) != CNULL) {
        buffer[MAX_LEN] = '\0'; /* Ensure NULL termination */
        neaten(buffer);
        if (*buffer != '\0') {
            ptr2=(struct member *) malloc(sizeof(struct member));
            ptr1->next      = ptr2;
            ptr2->previous  = ptr1;
            ptr2->next      = MNULL;
            strcpy(ptr1->data, buffer);
            ptr1 = ptr2;
            num++;
        }
    }
    
    /* Tidy up */
    if (num == 0) {
        ptr2 = ptr1;
        *start = MNULL;
    } else {
        ptr1 = ptr2->previous;
        ptr1->next = MNULL;
    }
    free(ptr2);
    return(num);
}

/* Look for the specified member in the list given and return the data */
PUBLIC int read_from_list(const struct member *start,
                        struct member *res, const char *target)
{
    struct member   *ptr1;

    if ((ptr1 = (struct member *) start) == MNULL)
        return(FALSE);  /* No list so not found */

    do {
        if (strcmp(ptr1->data,target) == 0) {
            if (res != MNULL)   /* If the data is wanted copy it out */
                memcpy(res,ptr1,sizeof(struct member));
            return(TRUE);   /* Fanfare! found it */
        }
        ptr1 = ptr1->next;
    } while (ptr1 != MNULL);

    return(FALSE);  /* Not there */
}

/* Look for the specified member in the list given, return TRUE/FALSE */
PUBLIC int find_in_list(const struct member *start, const char *target)
{
    return(read_from_list(start,MNULL,target));
}

/* Free up the memory associated with the list given */
PUBLIC void free_list(struct member **start)
{
    struct member   *ptr1,
                    *ptr2;

    if (*start == MNULL)
        return; /* Empty */

    ptr1 = *start;

    while (ptr1 != MNULL) {
        ptr2 = ptr1->next;
        free(ptr1);
        ptr1 = ptr2;
    }
    *start = MNULL;
}
